!*****************************************************************************
!                   Time-stamp: <2023-03-30 12:47:02 sander>
!      Author: Rolf Sander, Max-Planck Institute Mainz, Germany 1996-2014
!*****************************************************************************

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or (at
! your option) any later version. This program is distributed in the
! hope that it will be useful, but WITHOUT ANY WARRANTY; without even
! the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
! PURPOSE. See the GNU General Public License for more details. You
! should have received a copy of the GNU General Public License along
! with this program. If not, see: https://www.gnu.org/licenses/

!*****************************************************************************

MODULE henry_mem

  IMPLICIT NONE
  SAVE

  LOGICAL, PARAMETER :: L_MAKEINDEX = .FALSE. ! see xhenry

  CHARACTER(LEN=1), PARAMETER :: BACKSLASH = '\' !'

  INTEGER, PARAMETER :: STRLEN_CASRN    =    12
  INTEGER, PARAMETER :: STRLEN_INCHIKEY =    27
  INTEGER, PARAMETER :: STRLEN          =    30
  INTEGER, PARAMETER :: STRLEN_VLONG    =    80
  INTEGER, PARAMETER :: STRLEN_ULONG    =   256
  INTEGER, PARAMETER :: STRLEN_HUGE     = 20000

  ! I/O units
  ! reserved for reading input:   10
  INTEGER, PARAMETER :: IO_CONV = 21 ! tex
  INTEGER, PARAMETER :: IO_DATA = 22 ! tex
  INTEGER, PARAMETER :: IO_DEFS = 23 ! tex
  INTEGER, PARAMETER :: IO_NOTE = 24 ! tex
  INTEGER, PARAMETER :: IO_WARN = 30
  INTEGER, PARAMETER :: IO_ODBH = 31
  INTEGER, PARAMETER :: IO_ODBS = 32
  INTEGER, PARAMETER :: IO_DEBG = 33
  INTEGER, PARAMETER :: IO_SPCA = 34
  INTEGER, PARAMETER :: IO_SPCB = 35
  INTEGER, PARAMETER :: IO_SPCC = 36
  INTEGER, PARAMETER :: IO_PICS = 37
  INTEGER, PARAMETER :: IO_GNUP = 38 ! gnuplot
  INTEGER, PARAMETER :: IO_NML  = 41 ! namelist
  INTEGER, PARAMETER :: IO_PSQL = 42 ! psql
  INTEGER, PARAMETER :: IO_TDEP = 43 ! T-dependent data for regression (HTdep_ABC)
  ! f90 output files:
  INTEGER, PARAMETER :: IO_SOLUB  = 60 ! reserve 61...60+N_SOLUB
  INTEGER, PARAMETER :: IO_VOLAT  = 70 ! reserve 71...70+N_VOLAT

  ! molar masses [kg/mol]
  ! elements
  REAL, PARAMETER :: MH  =   1.00794E-3
  REAL, PARAMETER :: MC  =  12.011E-3
  REAL, PARAMETER :: MN  =  14.0067E-3
  REAL, PARAMETER :: MO  =  15.9994E-3
  REAL, PARAMETER :: MF  =  18.998E-3
  REAL, PARAMETER :: MNa =  22.98977E-3
  REAL, PARAMETER :: MMg =  24.305E-3
  REAL, PARAMETER :: MS  =  32.06E-3
  REAL, PARAMETER :: MCl =  35.453E-3
  REAL, PARAMETER :: MK  =  39.0983E-3
  REAL, PARAMETER :: MCa =  40.08E-3
  REAL, PARAMETER :: MBr =  79.904E-3
  REAL, PARAMETER :: MI  = 126.9045E-3
  ! compounds
  REAL, PARAMETER :: MH2O = MH*2.+MO

  ! composition of dry air:
  REAL, PARAMETER :: N2_fraction = 0.78084
  REAL, PARAMETER :: O2_fraction = 0.20946

  ! define some constants
  REAL, PARAMETER :: rhoH2O = 997.        ! density at T0 (CRC handbook) [kg/m3]
  REAL, PARAMETER :: cH2O   = rhoH2O/MH2O ! conc of H2O [mol/m3]
  REAL, PARAMETER :: p0     = 101325.     ! standard pressure [Pa]
  REAL, PARAMETER :: T0     = 298.15      ! standard temperature [K]
  REAL, PARAMETER :: Tstp   = 273.15      ! STP temperature [K]
  REAL, PARAMETER :: Rgas   = 8.3144621   ! gas constant [J/(mol*K)]

  ! define some non-SI conversion factors
  REAL, PARAMETER :: percent = 0.01
  REAL, PARAMETER :: kilo    = 1E3
  REAL, PARAMETER :: atm     = 101325.  ! [Pa]
  REAL, PARAMETER :: mmHg    = atm/760. ! [Pa] (mmHg=Torr)
  REAL, PARAMETER :: bar     = 1E5      ! [Pa]
  REAL, PARAMETER :: dm3     = 1E-3     ! [m3]
  REAL, PARAMETER :: cal     = 4.184    ! [J]
  REAL, PARAMETER :: kcal    = kilo*cal ! [J]
  REAL, PARAMETER :: CtoK    = 273.15

  ! conversion factors for Hs (solubilities)
  REAL, PARAMETER :: alpha_TO_HscpSI     = 1./(Rgas*Tstp) ! 4.403e-4
  REAL, PARAMETER :: S_TO_HscpSI         = 1E-3*rhoH2O/(Rgas*Tstp) ! for Kuenen in [cm3/g]
  REAL, PARAMETER :: Hsb2p_TO_Hsc2pSI    = rhoH2O**2/atm
  REAL, PARAMETER :: HsbpSI_TO_HscpSI    = rhoH2O
  REAL, PARAMETER :: Hsbp_TO_HscpSI      = rhoH2O/atm
  REAL, PARAMETER :: Hsc2p_TO_Hsc2pSI    = 1./(atm*dm3*dm3)
  !                  Hscc_TO_HscpSI      = see function definition
  REAL, PARAMETER :: Hscc_TO_HscpSI_atT0 = 1./(Rgas*T0)
  REAL, PARAMETER :: Hscp_TO_HscpSI      = 1./(atm*dm3)
  REAL, PARAMETER :: Hsxp_TO_HscpSI      = cH2O/atm

  ! products of Hs (solubilities) and Hv (volatilities)
  REAL, PARAMETER :: HvpcSI_TIMES_HscpSI    = 1.
  REAL, PARAMETER :: Hvpc_TIMES_HscpSI      = 1./atm
  REAL, PARAMETER :: Hvpx_TIMES_HscpSI      = cH2O/atm ! for Hvpx in [atm]
  !                  Hvcc_TO_HscpSI         = see function definition
  REAL, PARAMETER :: Hvcc_TIMES_HscpSI_atT0 = 1./(Rgas*T0)
  !Hvpw REAL, PARAMETER :: Hvpw_TIMES_HscpSI      = rhoH2O ! for Hvpw in [Pa] and M(solute) = 1 kg/mol

  !Hvpw INTEGER, PARAMETER :: N_SOLUB=7, N_VOLAT=5
  INTEGER, PARAMETER :: N_SOLUB=7, N_VOLAT=4
  ! conversion factors:
  REAL, PARAMETER, DIMENSION(N_SOLUB) :: conv_solub = (/ &
    1.,                       &  ! HscpSI to    HscpSI
    Hscp_TO_HscpSI,           &  ! Hscp   to    HscpSI
    Hscc_TO_HscpSI_atT0,      &  ! Hscc   to    HscpSI
    HsbpSI_TO_HscpSI,         &  ! HsbpSI to    HscpSI
    Hsbp_TO_HscpSI,           &  ! Hsbp   to    HscpSI
    Hsxp_TO_HscpSI,           &  ! Hsxp   to    HscpSI
    alpha_TO_HscpSI           /) ! alpha  to    HscpSI
  REAL, PARAMETER, DIMENSION(N_VOLAT) :: conv_volat = (/ &
    Hvpx_TIMES_HscpSI,        &  ! Hvpx   times HscpSI
    HvpcSI_TIMES_HscpSI,      &  ! HvpcSI times HscpSI
    HvpcSI_TIMES_HscpSI/atm,  &  ! Hvpc   times HscpSI
    Hvcc_TIMES_HscpSI_atT0    /) ! Hvcc   times HscpSI
    !Hvpw Hvpw_TIMES_HscpSI /) ! Hvpw   times HscpSI
  ! offset for mindHR:
  REAL, PARAMETER, DIMENSION(N_SOLUB) :: mindHR_offset = (/ &
    0., &  ! HscpSI
    0., &  ! Hscp
    T0, &  ! Hscc
    0., &  ! HsbpSI
    0., &  ! Hsbp
    0., &  ! Hsxp
    0.  /) ! alpha
  REAL, PARAMETER, DIMENSION(N_VOLAT) :: mindHR_inv_offset = (/ &
    0., &  ! Hvpx
    0., &  ! HvpcSI
    0., &  ! Hvpc
    T0  /) ! Hvcc
    !Hvpw 0.  /) ! Hvpw
  ! symbols (in TeX syntax):
  CHARACTER(LEN=STRLEN), DIMENSION(N_SOLUB) :: texsymbol_solub = (/ &
    "\Hscp ", &  ! HscpSI
    "\Hscp ", &  ! Hscp
    "\Hscc ", &  ! Hscc
    "\Hsbp ", &  ! HsbpSI
    "\Hsbp ", &  ! Hsbp
    "\Hsxp ", &  ! Hsxp
    "\alpha"  /) ! alpha
  CHARACTER(LEN=STRLEN), DIMENSION(N_VOLAT) :: texsymbol_volat = (/ &
    "\Hvpx", &  ! Hvpx
    "\Hvpc", &  ! HvpcSI
    "\Hvpc", &  ! Hvpc
    "\Hvcc"  /) ! Hvcc
    !Hvpw "\Hvpw"  /) ! Hvpw
  ! symbols (in plain text):
  CHARACTER(LEN=STRLEN), DIMENSION(N_SOLUB) :: txtsymbol_solub = (/ &
    "HscpSI ", &
    "Hscp   ", &
    "Hscc   ", &
    "HsbpSI ", &
    "Hsbp   ", &
    "Hsxp   ", &
    "alpha  "  /)
  CHARACTER(LEN=STRLEN), DIMENSION(N_VOLAT) :: txtsymbol_volat = (/ &
    "Hvpx  ", &
    "HvpcSI", &
    "Hvpc  ", &
    "Hvcc  "  /)
    !Hvpw "Hvpw  "  /)
  ! units shown as a fraction (in TeX syntax):
  CHARACTER(LEN=STRLEN), DIMENSION(N_SOLUB) :: unit_fr = (/ &
    "\DS\frac{mol}{m^3~Pa}", &  ! HscpSI
    "\DS\frac{M}{atm}     ", &  ! Hscp
    "                     ", &  ! Hscc
    "\DS\frac{mol}{kg~Pa} ", &  ! HsbpSI
    "\DS\frac{mol}{kg~atm}", &  ! Hsbp
    "\DS\frac{1}{atm}     ", &  ! Hsxp
    "                     "  /) ! alpha
  CHARACTER(LEN=STRLEN), DIMENSION(N_VOLAT) :: unit_fr_inv = (/ &
    "atm                   ", &  ! Hvpx
    "\DS\frac{m^3~Pa}{mol} ", &  ! HvpcSI
    "\DS\frac{m^3~atm}{mol}", &  ! Hvpc
    "                      "  /) ! Hvcc
    !Hvpw "Pa                    "  /) ! Hvpw
  ! units shown as one line (in TeX syntax):
  CHARACTER(LEN=STRLEN), DIMENSION(N_SOLUB) :: unit_ol = (/ &
    "mol/(m^3~Pa)", &  ! HscpSI
    "M/atm       ", &  ! Hscp
    "1           ", &  ! Hscc
    "mol/(kg~Pa) ", &  ! HsbpSI
    "mol/(kg~atm)", &  ! Hsbp
    "1/atm       ", &  ! Hsxp
    "1           "  /) ! alpha
  CHARACTER(LEN=STRLEN), DIMENSION(N_VOLAT) :: unit_ol_inv = (/ &
    "atm         ", &  ! Hvpx
    "m^3~Pa/mol  ", &  ! HvpcSI
    "m^3~atm/mol ", &  ! Hvpc
    "1           "  /) ! Hvcc
    !Hvpw "Pa          "  /) ! Hvpw
  CHARACTER :: lessthan, morethan

  ! global variables:
  REAL                            :: conv_fact ! conversion factor for output
  INTEGER                         :: ndata ! number of data points
  REAL, ALLOCATABLE, DIMENSION(:) :: temp, Harray ! array of size ndata
  REAL                            :: Hominus ! Henry constant at T0
  REAL                            :: mindHR ! - delta H / R
  REAL                            :: r2 ! correlation coefficient
  CHARACTER(STRLEN_ULONG)         :: current_catname, current_subcatname
  CHARACTER(STRLEN_VLONG)         :: chem ! name of the chemical species
  CHARACTER(STRLEN_CASRN)         :: casrn ! CAS registry number
  CHARACTER(STRLEN_INCHIKEY)      :: inchikey ! InChIKey
  CHARACTER(STRLEN_VLONG)         :: ref ! reference number in bib file
  CHARACTER(STRLEN_VLONG)         :: xref ! cross reference for type X
  CHARACTER                       :: type ! how value was obtained
  CHARACTER(STRLEN_ULONG)         :: seenote = "" ! for \seenote{}
  CHARACTER(STRLEN_HUGE)          :: AllLabels = "%" ! all \seenote{} labels
  CHARACTER(STRLEN_HUGE)          :: AllRefs = "%" ! all references
  INTEGER                         :: n_refs ! number of references
  LOGICAL                         :: warnings = .FALSE.
  LOGICAL                         :: l_testrun = .FALSE.
  ! global namelist variables:
  LOGICAL                         :: l_showpics     = .FALSE. ! show pictures?
  LOGICAL                         :: l_showbiblabel = .FALSE.
  LOGICAL                         :: l_debug        = .FALSE.
  LOGICAL                         :: l_verbose      = .FALSE.

  CHARACTER(STRLEN_VLONG)         :: H_symbol
  CHARACTER(STRLEN_VLONG)         :: H_unit_ol ! unit in one line, e.g.: M/atm
  CHARACTER(STRLEN_VLONG)         :: H_unit_fr ! fraction, e.g.: \frac{M}{atm}
  CHARACTER(STRLEN_VLONG)         :: Hprime_unit_ol_TeX

  TYPE HENRY_DATUM
    CHARACTER(STRLEN_VLONG)    :: chem     = "" ! name of the chemical species
    CHARACTER(STRLEN_CASRN)    :: casrn    = "" ! CAS registry number
    CHARACTER(STRLEN_INCHIKEY) :: inchikey = "" ! InChIKey
    REAL                       :: Hominus  = 0. ! Henry constant at T0
    REAL                       :: mindHR   = 0. ! - Delta H / R
    CHARACTER(STRLEN_VLONG)    :: ref      = "" ! reference number in bib file
    CHARACTER                  :: type     = "" ! how H was obtained
    CHARACTER(STRLEN_ULONG)    :: seenote  = "" ! for \seenote{}
    CHARACTER                  :: limit    = "" ! "<", ">" or "i"=infinity
  END TYPE HENRY_DATUM
  INTEGER, PARAMETER :: MAXDATA = 50000
  TYPE(HENRY_DATUM), DIMENSION(MAXDATA) :: henry_data
  INTEGER :: idx_h ! current index in henry_data array

  TYPE HENRY_NOTE
    CHARACTER(STRLEN_VLONG) :: label
    INTEGER :: start
    INTEGER :: END
  END TYPE HENRY_NOTE
  INTEGER, PARAMETER :: MAXNOTES = 1000
  TYPE(HENRY_NOTE), DIMENSION(MAXNOTES) :: henry_notes
  INTEGER :: idx_n ! current index in henry_notes array

  INTEGER, PARAMETER :: MAXNOTECHARS = 200000
  CHARACTER(MAXNOTECHARS) :: allnotes
  INTEGER :: idx_allnotes ! current index in allnotes

  ! dummies for undefined data:
  REAL, PARAMETER :: DUMMY = -999.
  REAL, PARAMETER :: NaN   = 1E99  ! "Not a Number" in input files

  INTEGER, PARAMETER :: MAXSPECIES = 12000
  CHARACTER(STRLEN_CASRN),    DIMENSION(MAXSPECIES) :: allcasrn
  CHARACTER(STRLEN_INCHIKEY), DIMENSION(MAXSPECIES) :: allinchikey
  INTEGER :: idx_s ! current index in allcasrn array

END MODULE Henry_mem

!*****************************************************************************
!                                  end of file
!*****************************************************************************
