#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# try to find "species" in the NIST webbook

# usage:
#   get_info_from_casrn.py > newspecies.f90

import re
import os, sys
import urllib.request
import requests

# pubchem api description:
# https://pubchem.ncbi.nlm.nih.gov/docs/pug-rest-tutorial
def from_pubchem(species, casrn):
    quoted_casrn = '"' + casrn + '"'
    response = requests.get('https://pubchem.ncbi.nlm.nih.gov/rest/pug/compound/name/' +
                            f'{casrn}/property/iupacname,MolecularFormula,InChIKey/JSON')
    results  = response.json()
    PropertyTable = results.get('PropertyTable')
    if (PropertyTable):
        Properties = PropertyTable.get('Properties')[0]
        name     = Properties.get('IUPACName', 'UNKNOWN')
        formula  = Properties.get('MolecularFormula', 'UNKNOWN')
        formula  = re.sub('([0-9]+)', '_{\\1}', formula) # convert to LaTeX
        formula  = re.sub('{([0-9])}', '\\1', formula) # remove curly braces for 1-digit numbers
        inchikey = Properties.get('InChIKey', '---------------------------')
        quoted_formula = '"' + formula + '",'
        print(f'  PUBCHEM CALL Species({quoted_formula:35} {quoted_casrn:>14}, "{inchikey}", "{name}", "") ! {species}')
    else:
        print(f'  ! NOT FOUND: {quoted_casrn:>14} "{species}"')

# python api description is from supporting information of:
# https://www.ncbi.nlm.nih.gov/pmc/articles/PMC9199008/
def from_commonchemistry(species, casrn):
    response  = requests.get('https://commonchemistry.cas.org/api/detail',
                             params={'cas_rn': casrn})
    results   = response.json()
    name      = results.get('name', 'UNKNOWN').lower()
    name      = name.replace('<em>', '{\\it ').replace('</em>', '}')
    formula   = results.get('molecularFormula', 'UNKNOWN')
    formula   = formula.replace('<sub>', '_{').replace('</sub>', '}')
    formula   = re.sub('{([0-9])}', '\\1', formula) # remove curly braces for 1-digit numbers
    inchikey  = results.get('inchiKey', '---------------------------')
    inchikey  = inchikey.replace('InChIKey=', '')
    casrn_out = results.get('rn', 'UNKNOWN')
    synonyms  = results.get('synonyms', 'UNKNOWN')
    #print(f'{name}\n{formula}\n{inchikey}\n{casrn_out}\n')
    quoted_casrn = '"' + casrn + '"'
    if (casrn == casrn_out):
        quoted_formula = '"' + formula + '",'
        print(f'  COMMON  CALL Species({quoted_formula:35} {quoted_casrn:>14}, "{inchikey}", "{name}", "") ! {species}')
    else:
        print(f'  ! NOT FOUND: {quoted_casrn:>14} "{species}"')

def from_chemicalbook(species, casrn):
    url = f'https://m.chemicalbook.com/Search_EN.aspx?keyword={casrn}'
    name      = 'UNKNOWN'
    formula   = 'UNKNOWN'
    inchikey  = '---------------------------'
    casrn_out = 'UNKNOWN'
    with urllib.request.urlopen(url) as f:
        for line in f:
            line = line.decode("utf-8").strip()
            if ('Chemical Name' in line):
                name = re.sub('.*</span>', '', line)
                name = re.sub('</li>.*', '', name)
                name = name.lower()
            if ('Molecular Formula' in line):
                formula = re.sub('.*</span>', '', line)
                formula = re.sub('</li>.*', '', formula)
                formula = re.sub('([0-9]+)', '_{\\1}', formula) # convert to LaTeX
                formula = re.sub('{([0-9])}', '\\1', formula) # remove curly braces for 1-digit numbers
            if ('CAS No' in line):
                casrn_out = re.sub('.*</span>', '', line)
                casrn_out = re.sub('</li>.*', '', casrn_out)
        quoted_casrn = '"' + casrn + '"'
        if (casrn == casrn_out):
            quoted_formula = '"' + formula + '",'
            print(f'  CHBOOK  CALL Species({quoted_formula:35} {quoted_casrn:>14}, "{inchikey}", "{name}", "") ! {species}')
        else:
            print(f'  ! NOT FOUND: {quoted_casrn:>14} "{species}"')

def from_nist(species, casrn):
    url = f'https://webbook.nist.gov/cgi/cbook.cgi?ID={casrn}'
    name      = 'UNKNOWN'
    formula   = 'UNKNOWN'
    inchikey  = '---------------------------'
    casrn_out = 'UNKNOWN'
    with urllib.request.urlopen(url) as f:
        for line in f:
            line = line.decode("utf-8").strip()
            if ('id="Top"' in line):
                name = re.sub('.*id="Top">', '', line)
                name = re.sub('</h1>.*', '', name)
                name = name.lower()
            if ('http://goldbook.iupac.org/E02063.html' in line):
                formula = re.sub('.*</strong> ', '', line)
                formula = re.sub('</li>.*', '', formula)
                formula = formula.replace('<sub>', '_{').replace('</sub>', '}')
                formula = re.sub('{([0-9])}', '\\1', formula) # remove curly braces for 1-digit numbers
            if ('class="inchi-text"' in line):
                inchikey = re.sub('.*class="inchi-text">', '', line)
                inchikey = re.sub('</span>.*', '', inchikey)
            if ('CAS Registry' in line):
                casrn_out = re.sub('.*</strong> ', '', line)
                casrn_out = re.sub('</li>.*', '', casrn_out)
        quoted_casrn = '"' + casrn + '"'
        if (casrn == casrn_out):
            quoted_formula = '"' + formula + '",'
            print(f'  NIST    CALL Species({quoted_formula:35} {quoted_casrn:>14}, "{inchikey}", "{name}", "") ! {species}')
        else:
            print(f'  ! NOT FOUND: {quoted_casrn:>14} "{species}"')

def create_CALL_Species(species, casrn):
    from_commonchemistry(species, casrn)
    from_pubchem(species, casrn)
    from_nist(species, casrn)
    from_chemicalbook(species, casrn)
    print()

if __name__ == '__main__':
    create_CALL_Species("gamma-neoclovene", "")
