#!/usr/bin/env python3
# -*- coding: utf-8 -*- Time-stamp: <2023-03-28 15:22:38 sander>

# perform some tests to check that alpha, l, and q data are consistent

import numpy as np
import matplotlib.pyplot as plt
from henry_util import CtoK, p0, alpha_TO_HscpSI, rhoH2O, psat

MCl2 = 70.906  # M(Cl2) in g/mol
MCH4 = 16.0425 # M(CH4) in g/mol

def qtol(M,T):
    # conversion factor from q to l
    return 10 * rhoH2O(T) / (p0*M*alpha_TO_HscpSI)

def ltoalpha(T):
    # conversion factor from l to alpha
    return p0 / (p0-psat(T))
    

if __name__ == '__main__':

    # -------------------------------------------------------

    print('chlorine')

    # T, l, and q from Tab. 5.1 (don't use value at 100 C because it is 0):
    T = np.array([10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22,
                  23, 24, 25, 26, 27, 28, 29, 30, 35, 40, 45, 50, 60,
                  70, 80, 90]) + CtoK
    
    l = np.array([3.148, 3.047, 2.950, 2.856, 2.767, 2.680, 2.597, 2.517,
                  2.440, 2.368, 2.299, 2.238, 2.180, 2.123, 2.070, 2.019, 1.970, 1.923,
                  1.880, 1.839, 1.799, 1.602, 1.438, 1.322, 1.225, 1.023, 0.862, 0.683,
                  0.39])
     
    q = np.array([0.9972, 0.9654, 0.9346, 0.9050, 0.8768, 0.8495, 0.8232,
                  0.7979, 0.7738, 0.7510, 0.7293, 0.7100, 0.6918, 0.6739, 0.6572, 0.6413,
                  0.6259, 0.6112, 0.5975, 0.5847, 0.5723, 0.5104, 0.4590, 0.4228, 0.3925,
                  0.3295, 0.2793, 0.2227, 0.127])

    # calculate l from q:
    l2 = q * qtol(MCl2,T)
    
    # compare l from Tab. 5.1 and l calculated from q:
    for i in range(len(T)):
        print(f'{T[i]:.2f} {l[i]:.7f} {l2[i]:.7f} {l[i]/l2[i]:.7f}')
    
    plt.figure()
    plt.plot(T, l/(q*qtol(MCl2,T)))
    plt.savefig('ref0379_Cl2.pdf')
    
    # -------------------------------------------------------

    print('methane')

    # T, alpha, and q from Tab. 5.1 (don't use value at 100 C because it is 0):
    T = np.array([0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15,
                  16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28,
                  29, 30, 35, 40, 45, 50, 60, 70, 80, 90]) + CtoK    
    alpha = np.array([0.05563, 0.05401, 0.05244, 0.05093, 0.04946,
                      0.04805, 0.04669, 0.04539, 0.04413, 0.04292, 0.04177, 0.04072,
                      0.03970, 0.03872, 0.03779, 0.03690, 0.03606, 0.03525, 0.03448,
                      0.03376, 0.03308, 0.03243, 0.03180, 0.03119, 0.03061, 0.03006,
                      0.02952, 0.02901, 0.02852, 0.02806, 0.02762, 0.02546, 0.02369,
                      0.02238, 0.02134, 0.01954, 0.01825, 0.01770, 0.01735])
    q = np.array([0.003959, 0.003842, 0.003728, 0.003619, 0.003513,
                  0.003410, 0.003312, 0.003217, 0.003127, 0.003039, 0.002955,
                  0.002879, 0.002805, 0.002733, 0.002665, 0.002599, 0.002538,
                  0.002478, 0.002422, 0.002369, 0.002319, 0.002270, 0.002222, 0.002177,
                  0.002133, 0.002091, 0.002050, 0.002011, 0.001974, 0.001938,
                  0.001904, 0.001733, 0.001586, 0.001466, 0.001359, 0.001144,
                  0.000926, 0.000695, 0.00040])

    # calculate l from q:
    l = q * qtol(MCH4,T)
    # calculate alpha from l:
    alpha2 = l * ltoalpha(T)

    # compare alpha from Tab. 5.1 and alpha calculated from q:
    for i in range(len(T)):
        print(f'{T[i]:.2f} {alpha[i]:.7f} {alpha2[i]:.7f} {alpha[i]/alpha2[i]:.7f}')
    
    plt.figure()
    plt.plot(T, alpha/alpha2)
    #plt.plot(T, alpha/(q*qtol(MCH4,T)*ltoalpha(T)))
    plt.savefig('ref0379_CH4.pdf')
    
    # -------------------------------------------------------

    print('HBr')

    T = np.array([0., 10., 25., 50., 75.]) + CtoK    
    l = np.array([612., 582., 533., 469., 406. ])
                   
    # calculate q from l:
    q = l / qtol(MCH4,T)
    # calculate alpha from l:
    alpha = l * ltoalpha(T)
    # calculate Hscp from alpha:
    Hscp = alpha * alpha_TO_HscpSI

    for i in range(len(T)):
        print(f'{T[i]:.2f} {Hscp[i]:.7f} {alpha[i]:.7f} {q[i]:.7f} {l[i]:.7f}')
 
    # -------------------------------------------------------
